﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Net;
using System.Net.Sockets;
using NAudio.Wave;
using NAudio.CoreAudioApi;
using System.Runtime.InteropServices;
using System.Threading;

namespace PIAudioClient
{
    class Audio
    {
        private const int PACKET_SIZE = 19200;

        private String address;
        private int port;

        private int volume = 80;

        private TcpClient client;
        private NetworkStream stream;
        WasapiLoopbackCapture sourceStream;

        private byte[] packet = new byte[PACKET_SIZE];

        private Boolean isRunning;

        public Audio(String address, int port)
        {
            this.address = address;
            this.port = port;
        }

        public void start()
        {
            isRunning = true;
            _start();
        }

        private void _start()
        {
            try {
                setMute(true);

                client = new TcpClient();
                //client.NoDelay = true;
                client.SendBufferSize = PACKET_SIZE;
                client.Connect(address, port);
                stream = client.GetStream();

                sourceStream = new NAudio.Wave.WasapiLoopbackCapture();
                sourceStream.DataAvailable += new EventHandler<NAudio.Wave.WaveInEventArgs>(DataAvailable);
                sourceStream.StartRecording();                
            }
            catch (Exception)
            {
                _stop();
            }
            
        }

        public void stop()
        {
            isRunning = false;
            _stop();
        }

        private void _stop()
        {
            try
            {
                setMute(false);
                sourceStream.StopRecording();
                sourceStream.Dispose();
                stream.Close();
                stream.Dispose();
                client.Close();
            }
            catch (Exception) { }
        }

        private static void setMute(Boolean value)
        {
             MMDeviceEnumerator devEnum = new MMDeviceEnumerator();
             MMDevice defaultDevice = devEnum.GetDefaultAudioEndpoint(DataFlow.Render, Role.Multimedia);
             defaultDevice.AudioEndpointVolume.Mute = value;
        }

        public void setVolume(int volume)
        {
            this.volume = volume;
        }

        public int getVolume()
        {
            return this.volume;
        }

        private void DataAvailable(object sender, NAudio.Wave.WaveInEventArgs e)
        {
           if (!client.Connected) return;

            int bufferIncrement = (int) (sourceStream.WaveFormat.BlockAlign / sourceStream.WaveFormat.Channels);
            int packetCounter = 0;

            for (int index = 0; index < e.BytesRecorded; index += bufferIncrement)
            {
                float sample32 = BitConverter.ToSingle(e.Buffer, index);
                short sample16 = (short)(sample32 * (32768f * (volume / 100f)));

                if (packetCounter == packet.Length)
                {
                    send(packet, 0, packetCounter);
                    packetCounter = 0;
                }
                else
                {
                    packet[packetCounter] = (byte) (sample16 & 0xFF);
                    packet[packetCounter + 1] = (byte) (sample16 >> 8);
                    packetCounter += 2;
                }
            }

            if (packetCounter != 0)
                send(packet, 0, packetCounter);
        }

        public void send(byte[] packet, int offset, int length)
        {
            try
            {
                stream.Write(packet, offset, length);
            }
            catch (Exception)
            {
                new Thread(delegate()
                {
                    if (!isRunning)
                        return;
                    _stop();
                    _start();
                }).Start();
            }
        }
    }
}
