﻿using PIAudioClient.Properties;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace PIAudioClient
{
    public partial class Main : Form
    {

        private int PORT = 10000;
        private String ADDRESS = "127.0.0.1";

        private NotifyIcon trayIcon;
        private MenuItem buttonEnabled;

        private Boolean isRunning = false;
        private Audio audio;

        public Main(string[] args)
        {
            InitializeComponent();

            for(int i = 0; i < args.Length - 1; i++)
            {
                if (args[i].CompareTo("--address") == 0)
                {
                    ADDRESS = args[i + 1];
                }
                else if (args[i].CompareTo("--port") == 0)
                {
                    PORT = Convert.ToInt32(args[i + 1]);
                }
            }

            audio = new Audio(ADDRESS, PORT);

            ContextMenu trayMenu = new ContextMenu();
            buttonEnabled = trayMenu.MenuItems.Add("Volume", openVolume);
            buttonEnabled = trayMenu.MenuItems.Add("Enabled", toggleRunning);
            trayMenu.MenuItems.Add("Exit", OnExit);
 
            trayIcon      = new NotifyIcon();
            trayIcon.Text = "PIAudioClient";
            trayIcon.Icon = Resources.icon2;
            trayIcon.DoubleClick += new EventHandler(openVolume);
 
            trayIcon.ContextMenu = trayMenu;
            trayIcon.Visible     = true;
        }

        private void hide(object sender, EventArgs e)
        {
            Visible = false;
        }

        private void openVolume(object sender, EventArgs e) 
        {
            Point cursor = Cursor.Position;
            StartPosition = FormStartPosition.Manual;
            Location = new Point(cursor.X - Width / 2, cursor.Y - Height - 10);
            Visible = true;
            Activate();
            volume.Focus();
            
        }

        private void toggleRunning(object sender, EventArgs e)
        {
            if (isRunning)
            {
                audio.stop();
                buttonEnabled.Checked = false;
                isRunning = false;
                trayIcon.Icon = Resources.icon2;
            }
            else
            {
                audio.start();
                buttonEnabled.Checked = true;
                isRunning = true;
                trayIcon.Icon = Resources.icon1;
            }
        }
 
        protected override void OnLoad(EventArgs e)
        {
            Visible       = false;
            ShowInTaskbar = false; 
            volume.Value  = audio.getVolume();
            base.OnLoad(e);
        }
 
        private void OnExit(object sender, EventArgs e)
        {
            audio.stop();
            Application.Exit();
        }
 
        protected override void Dispose(bool isDisposing)
        {
            if (isDisposing)
            {
                trayIcon.Dispose();
            }
            if (isDisposing && (components != null))
            {
                components.Dispose();
            }
            base.Dispose(isDisposing);
        }

        private void volume_Scroll(object sender, EventArgs e)
        {
            audio.setVolume(volume.Value);
            volumeLabel.Text = volume.Value + "%";

        }


    }
}
